/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.modelsconference;

import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_A_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_B_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_C_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_D_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_LEFT_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_RIGHT_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LCD_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LED_LEFT_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LED_RED_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LED_RIGHT_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LED_WHITE_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.POTI_LEFT_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.POTI_RIGHT_UID;
import static org.fortiss.af3.platform.modelsconference.model.PlatformModelsConferenceFactory.eINSTANCE;
import static org.fortiss.af3.platform.raspberry.util.RaspberryModelElementFactory.createRaspberryPi;
import static org.fortiss.tooling.base.model.layout.EOrientation.EAST;
import static org.fortiss.tooling.base.model.layout.EOrientation.NORTH;
import static org.fortiss.tooling.base.model.layout.EOrientation.SOUTH;
import static org.fortiss.tooling.base.model.layout.EOrientation.WEST;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.setNodePosition;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.setNodeSize;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.setStickyConnectorLayoutData;
import static org.fortiss.tooling.base.utils.LayoutModelElementFactory.createConnectorLayout;

import org.fortiss.af3.platform.modelsconference.model.UIDBase;
import org.fortiss.af3.platform.raspberry.model.RaspberryPi;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.layout.EOrientation;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.kernel.model.INamedElement;

/**
 * Ḿodel element factory for the models conference platform.
 * 
 * @author hoelzl
 */
public final class ModelsConferencePlatformElementFactory {
	/** Creates the demonstrator board with all sensor and actuators. */
	public static RaspberryPi createDemonstratorBoard() {
		RaspberryPi pi = createRaspberryPi();
		setNodePosition(pi, 160, 80);
		setNodeSize(pi, 320, 400);
		pi.setName("Models Conference Board");
		pi.setCoordinatorUnit(true);

		addConnector(pi, eINSTANCE.createButton(), WEST, 36, "ButtonLeft", BUTTON_LEFT_UID);
		addConnector(pi, eINSTANCE.createButton(), EAST, 36, "ButtonRight", BUTTON_RIGHT_UID);
		addConnector(pi, eINSTANCE.createButton(), EAST, 120, "ButtonA", BUTTON_A_UID);
		addConnector(pi, eINSTANCE.createButton(), EAST, 156, "ButtonB", BUTTON_B_UID);
		addConnector(pi, eINSTANCE.createButton(), EAST, 192, "ButtonC", BUTTON_C_UID);
		addConnector(pi, eINSTANCE.createButton(), EAST, 228, "ButtonD", BUTTON_D_UID);

		addConnector(pi, eINSTANCE.createLCD(), NORTH, 132, "LCD", LCD_UID);

		addConnector(pi, eINSTANCE.createLED(), WEST, 54, "LEDLeft", LED_LEFT_UID);
		addConnector(pi, eINSTANCE.createLED(), EAST, 54, "LEDRight", LED_RIGHT_UID);
		addConnector(pi, eINSTANCE.createLED(), SOUTH, 36, "LEDRed", LED_RED_UID);
		addConnector(pi, eINSTANCE.createLED(), SOUTH, 192, "LEDWhite", LED_WHITE_UID);

		addConnector(pi, eINSTANCE.createPoti(), WEST, 324, "PotiLeft", POTI_LEFT_UID);
		addConnector(pi, eINSTANCE.createPoti(), EAST, 324, "PotiRight", POTI_RIGHT_UID);

		return pi;
	}

	/** Adds the given connector with layout data, name and unique id. */
	private static <T extends INamedElement & UIDBase & IConnector & ILayoutedModelElement> void
			addConnector(RaspberryPi pi, T connector, EOrientation eorient, int offset,
					String name, String uid) {
		connector.setName(name);
		connector.setUid(uid);
		createConnectorLayout(connector);
		setStickyConnectorLayoutData(connector, eorient, offset);
		pi.getConnectors().add(connector);
	}
}
