/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.modelsconference.generator;

import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_A_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_B_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_C_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_D_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_LEFT_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.BUTTON_RIGHT_UID;

import org.fortiss.af3.component.model.InputPort;
import org.fortiss.af3.platform.modelsconference.model.Button;
import org.fortiss.af3.platform.raspberry.generator.framework.IReadableGeneratorExtension;

/**
 * Generator extension for {@link Button}.
 * 
 * @author hoelzl
 */
public final class ButtonGeneratorExtension extends WiringPiLibraryGeneratorExtensionBase<Button>
		implements IReadableGeneratorExtension {
	/** Constructor. */
	public ButtonGeneratorExtension(Button modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceInitializationCode() {
		String pin = getPinNumber();
		return "pinMode(" + pin + ", INPUT);\n" + "pullUpDnControl(" + pin + ", PUD_UP);\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getReadCode(InputPort logicalSignal, String targetVariable) {
		return targetVariable + " = digitalRead(" + getPinNumber() + ") ? 0 : 1;\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getNoValReadCode(InputPort logicalSignal) {
		return null;
	}

	/** Returns the pin number for the given button UID. */
	private String getPinNumber() {
		String btnUID = modelElement.getUid();
		if(BUTTON_LEFT_UID.equals(btnUID)) {
			return "27";
		}
		if(BUTTON_RIGHT_UID.equals(btnUID)) {
			return "26";
		}
		if(BUTTON_A_UID.equals(btnUID)) {
			return "6";
		}
		if(BUTTON_B_UID.equals(btnUID)) {
			return "5";
		}
		if(BUTTON_C_UID.equals(btnUID)) {
			return "4";
		}
		if(BUTTON_D_UID.equals(btnUID)) {
			return "1";
		}
		return "UNKNOWN_MODEL_ELEMENT_UID";
	}
}
