/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.modelsconference.generator;

import static java.util.Arrays.asList;

import java.util.List;

import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.platform.modelsconference.model.LCD;
import org.fortiss.af3.platform.raspberry.generator.framework.IWriteableGeneratorExtension;

/**
 * Generator extension for {@link LCD}.
 * 
 * @author hoelzl
 */
public final class LCDGeneratorExtension extends WiringPiLibraryGeneratorExtensionBase<LCD>
		implements IWriteableGeneratorExtension {
	/** Constructor. */
	public LCDGeneratorExtension(LCD modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public List<String> getHeaderIncludes() {
		return asList("OLED_Display.h");
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonIdentifier() {
		return "OLED_Display.h";
	}

	/** {@inheritDoc} */
	@Override
	public void setSingletonPostfix(String singletonPostfix) {
		this.singletonPostfix = singletonPostfix == null ? "" : singletonPostfix;
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonVariableDeclarationCode() {
		return "char oled_text[50];\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonInitializationCode() {
		return "initDisplay();\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getWriteCode(OutputPort logicalSignal, String value) {
		return "clearDisplay();\nsprintf(oled_text, \"Speed: %G km/h\", " + value +
				");\nprintTextPos(oled_text, 0, 0);\nshowDisplay();\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getNoValWriteCode(OutputPort logicalSignal) {
		return "clearDisplay();\nsprintf(oled_text, \"Speed: 0.0 km/h\");\nprintTextPos(oled_text, 0, 0);\nshowDisplay();\n";
	}
}
