/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.modelsconference.generator;

import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LED_LEFT_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LED_RED_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LED_RIGHT_UID;
import static org.fortiss.af3.platform.modelsconference.UniqueIdentifiers.LED_WHITE_UID;

import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.platform.modelsconference.model.LED;
import org.fortiss.af3.platform.raspberry.generator.framework.IInstanceGeneratorExtension;
import org.fortiss.af3.platform.raspberry.generator.framework.IWriteableGeneratorExtension;

/**
 * Generator extension for {@link LED}.
 * 
 * @author hoelzl
 */
public final class LEDGeneratorExtension extends WiringPiLibraryGeneratorExtensionBase<LED>
		implements IInstanceGeneratorExtension, IWriteableGeneratorExtension {
	/** Constructor. */
	public LEDGeneratorExtension(LED modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceInitializationCode() {
		String pin = getPinNumber();
		return "pinMode(" + pin + ", OUTPUT);\n" + "digitalWrite(" + pin + ", HIGH);\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getWriteCode(OutputPort logicalSignal, String value) {
		String pin = getPinNumber();
		return "digitalWrite(" + pin + ", (" + value + " == 1 ? HIGH : LOW));\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getNoValWriteCode(OutputPort logicalSignal) {
		String pin = getPinNumber();
		return "digitalWrite(" + pin + ", LOW);\n";
	}

	/** Returns the pin number for the given button UID. */
	private String getPinNumber() {
		String btnUID = modelElement.getUid();
		if(LED_LEFT_UID.equals(btnUID)) {
			return "23";
		}
		if(LED_RIGHT_UID.equals(btnUID)) {
			return "7";
		}
		if(LED_RED_UID.equals(btnUID)) {
			return "2";
		}
		if(LED_WHITE_UID.equals(btnUID)) {
			return "3";
		}
		return "UNKNOWN_MODEL_ELEMENT_UID";
	}
}
