/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.modelsconference.generator;

import static java.util.Arrays.asList;

import java.util.List;

import org.fortiss.af3.component.model.InputPort;
import org.fortiss.af3.platform.modelsconference.model.Poti;
import org.fortiss.af3.platform.raspberry.generator.framework.IReadableGeneratorExtension;

/**
 * Generator extension for {@link Poti}.
 * 
 * @author hoelzl
 */
public final class PotiGeneratorExtension extends WiringPiLibraryGeneratorExtensionBase<Poti>
		implements IReadableGeneratorExtension {
	/** Constructor. */
	public PotiGeneratorExtension(Poti modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public List<String> getHeaderIncludes() {
		return asList("ABE_ADCPi.c");
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonIdentifier() {
		return "ABE_ADCPi.c";
	}

	/** {@inheritDoc} */
	@Override
	public void setSingletonPostfix(String singletonPostfix) {
		this.singletonPostfix = singletonPostfix == null ? "" : singletonPostfix;
	}

	/** {@inheritDoc} */
	@Override
	public String getReadCode(InputPort logicalSignal, String targetVariable) {
		return targetVariable + " = read_voltage(0x68, " +
				(targetVariable.contains("Left") ? "2" : "1") + ", 12, 1, 1);\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getNoValReadCode(InputPort logicalSignal) {
		return null;
	}
}
