/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.modelsconference.generator;

import static java.util.Arrays.asList;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.platform.language.executable.ExecutableBase;
import org.fortiss.af3.platform.raspberry.generator.framework.IInstanceGeneratorExtension;
import org.fortiss.af3.platform.raspberry.generator.framework.IPreinstalledHeaderGeneratorExtension;
import org.fortiss.af3.platform.raspberry.generator.framework.IPreinstalledLibraryGeneratorExtension;
import org.fortiss.af3.platform.raspberry.generator.framework.ISingletonGeneratorExtension;

/**
 * 
 * @author hoelzl
 */
abstract class WiringPiLibraryGeneratorExtensionBase<T extends EObject> extends ExecutableBase<T>
		implements IPreinstalledLibraryGeneratorExtension, IPreinstalledHeaderGeneratorExtension,
		ISingletonGeneratorExtension, IInstanceGeneratorExtension {
	/** Stores the singleton postfix. */
	protected String singletonPostfix = "";
	/** Stores the instance postfix. */
	protected String instancePostfix = "";

	/** Constructor. */
	public WiringPiLibraryGeneratorExtensionBase(T modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public List<String> getHeaderIncludes() {
		return asList("wiringPi.h", "wiringPiI2C.h");
	}

	/** {@inheritDoc} */
	@Override
	public List<String> getLibraryNames() {
		return asList("wiringPi");
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonIdentifier() {
		return "wiringPi.h";
	}

	/** {@inheritDoc} */
	@Override
	public void setSingletonPostfix(String singletonPostfix) {
		this.singletonPostfix = singletonPostfix == null ? "" : singletonPostfix;
	}

	/** {@inheritDoc} */
	@Override
	public void setInstancePostfix(String instancePostfix) {
		this.instancePostfix = instancePostfix == null ? "" : instancePostfix;
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonVariableDeclarationCode() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceVariableDeclaration() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonAuxiliaryFunctions() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceAuxiliaryFunctions() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonInitializationCode() {
		return "	if (wiringPiSetup() == -1) {\n" + "exit(-1);\n" + "}\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceInitializationCode() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonTerminationCode() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getInstanceTerminationCode() {
		return null;
	}
}
